import 'package:firebase_auth/firebase_auth.dart' show FirebaseAuth;
import 'package:flutter/widgets.dart';
import 'package:flutterfire_ui/auth.dart';
import 'package:flutterfire_ui/i10n.dart';

import '../widgets/internal/universal_button.dart';
import '../widgets/internal/universal_page_route.dart';
import '../widgets/internal/universal_scaffold.dart';

import 'internal/responsive_page.dart';

/// A screen displaying a fully styled phone number entry screen, with a country-code
/// picker.
///
/// {@subCategory service:auth}
/// {@subCategory type:screen}
/// {@subCategory description:A screen displaying a fully styled phone number entry input with a country-code picker.}
/// {@subCategory img:https://place-hold.it/400x150}
class PhoneInputScreen extends StatelessWidget {
  final AuthAction? action;
  final FirebaseAuth? auth;
  final WidgetBuilder? subtitleBuilder;
  final WidgetBuilder? footerBuilder;
  final HeaderBuilder? headerBuilder;
  final double? headerMaxExtent;
  final SideBuilder? sideBuilder;
  final TextDirection? desktopLayoutDirection;

  const PhoneInputScreen({
    Key? key,
    this.action,
    this.auth,
    this.subtitleBuilder,
    this.footerBuilder,
    this.headerBuilder,
    this.headerMaxExtent,
    this.sideBuilder,
    this.desktopLayoutDirection,
  }) : super(key: key);

  void next(BuildContext context, AuthAction? action, Object flowKey, _) {
    Navigator.of(context).push(
      createPageRoute(
        context: context,
        builder: (context) => SMSCodeInputScreen(
          action: action,
          flowKey: flowKey,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final flowKey = Object();
    final l = FirebaseUILocalizations.labelsOf(context);

    return UniversalScaffold(
      body: ResponsivePage(
        desktopLayoutDirection: desktopLayoutDirection,
        sideBuilder: sideBuilder,
        headerBuilder: headerBuilder,
        headerMaxExtent: headerMaxExtent,
        breakpoint: 400,
        child: Padding(
          padding: const EdgeInsets.all(20),
          child: Column(
            children: [
              PhoneInputView(
                auth: auth,
                action: action,
                subtitleBuilder: subtitleBuilder,
                footerBuilder: footerBuilder,
                flowKey: flowKey,
                onSMSCodeRequested: next,
              ),
              UniversalButton(
                text: l.goBackButtonLabel,
                variant: ButtonVariant.text,
                onPressed: () {
                  Navigator.of(context).pop();
                },
              ),
            ],
          ),
        ),
      ),
    );
  }
}
